@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2020-2021 Valve Corporation
- * Copyright (c) 2020-2021 LunarG, Inc.
+ * Copyright (c) 2020-2024 Valve Corporation
+ * Copyright (c) 2020-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,19 +52,7 @@ typedef enum _pe_architecture {
     PE_ARCHITECTURE_x64 = 0x020B
 } PE_ARCHITECTURE;
 
-LPVOID GetOffsetFromRva(IMAGE_DOS_HEADER *pDos, IMAGE_NT_HEADERS *pNt, DWORD rva) {
-    IMAGE_SECTION_HEADER *pSecHd = IMAGE_FIRST_SECTION(pNt);
-    for (unsigned long i = 0; i < pNt->FileHeader.NumberOfSections; ++i, ++pSecHd) {
-        // Lookup which section contains this RVA so we can translate the VA to a file offset
-        if (rva >= pSecHd->VirtualAddress && rva < (pSecHd->VirtualAddress + pSecHd->Misc.VirtualSize)) {
-            DWORD delta = pSecHd->VirtualAddress - pSecHd->PointerToRawData;
-            return (LPVOID)MKPTR(pDos, rva - delta);
-        }
-    }
-    return NULL;
-}
-
-PE_ARCHITECTURE GetImageArchitecture(void *pImageBase) {
+static PE_ARCHITECTURE GetImageArchitecture(void *pImageBase) {
     // Parse and validate the DOS header
     IMAGE_DOS_HEADER *pDosHd = (IMAGE_DOS_HEADER *)pImageBase;
     if (IsBadReadPtr(pDosHd, sizeof(pDosHd->e_magic)) || pDosHd->e_magic != IMAGE_DOS_SIGNATURE) return PE_ARCHITECTURE_UNKNOWN;
@@ -120,9 +108,10 @@ LayersDialog::LayersDialog(QWidget *parent, const Configuration &configuration)
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     Configurator &configurator = Configurator::Get();
-    configurator.configurations.SaveAllConfigurations(configurator.layers.available_layers);
+    configurator.configurations.SaveAllConfigurations(configurator.layers.selected_layers);
 
     ui->lineEditName->setText(configuration.key.c_str());
+    ui->lineEditName->setFocus();
     ui->lineEditDescription->setText(configuration.description.c_str());
     ui->buttonBox->setEnabled(!configurator.layers.Empty());
     ui->pushButtonRemove->setEnabled(false);
@@ -199,11 +188,12 @@ void LayersDialog::Reload() {
     std::vector<std::string> user_defined_paths = this->configuration.user_defined_paths;
 
     Configurator &configurator = Configurator::Get();
+    configurator.configurations.SaveAllConfigurations(configurator.layers.selected_layers);
     configurator.configurations.available_configurations.clear();
     configurator.environment.SetPerConfigUserDefinedLayersPaths(this->configuration.user_defined_paths);
 
     configurator.layers.LoadAllInstalledLayers();
-    configurator.configurations.LoadAllConfigurations(configurator.layers.available_layers);
+    configurator.configurations.LoadAllConfigurations(configurator.layers.selected_layers);
 
     Configuration *saved_configuration =
         FindByKey(configurator.configurations.available_configurations, configuration_name.c_str());
@@ -215,8 +205,8 @@ void LayersDialog::Reload() {
     RestoreParameterStates(this->configuration.parameters, ParameterStates);
     this->configuration.user_defined_paths = user_defined_paths;
 
-    configurator.configurations.RefreshConfiguration(configurator.layers.available_layers);
-    configurator.request_vulkan_status = true;
+    configurator.environment.SetSelectedConfiguration(configuration_name);
+    configurator.configurations.Configure(configurator.layers.selected_layers);
 }
 
 void LayersDialog::UpdateUI() {
@@ -270,7 +260,7 @@ void LayersDialog::OnLayerTreeSortedClicked(QTreeWidgetItem *item, int column) {
 void LayersDialog::AddLayerItem(const Parameter &parameter) {
     assert(!parameter.key.empty());
 
-    std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
+    std::vector<Layer> &available_layers = Configurator::Get().layers.selected_layers;
 
     const Layer *layer = FindByKey(available_layers, parameter.key.c_str());
 
@@ -303,7 +293,9 @@ void LayersDialog::AddLayerItem(const Parameter &parameter) {
     TreeWidgetItemParameter *item = new TreeWidgetItemParameter(parameter.key.c_str());
 
     item->setText(0, decorated_name.c_str());
-    if (layer != nullptr) item->setToolTip(0, layer->manifest_path.c_str());
+    if (layer != nullptr) {
+        item->setToolTip(0, layer->manifest_path.c_str());
+    }
     item->setFlags(item->flags() | Qt::ItemIsSelectable);
     item->setDisabled(layer == nullptr);
 
@@ -314,16 +306,40 @@ void LayersDialog::AddLayerItem(const Parameter &parameter) {
     WidgetTreeFriendlyComboBox *widget = new WidgetTreeFriendlyComboBox(item);
     ui->layerTree->setItemWidget(item, 1, widget);
 
-    const QFontMetrics fm = ui->layerTree->fontMetrics();
-    const QSize combo_size = fm.size(Qt::TextSingleLine, "Application-Controlled") * 1.6;
-    item->setSizeHint(1, combo_size);
+    if (layer != nullptr) {
+        std::string tooltip;
+
+        bool disable_value = false;
+        bool enable_value = false;
+
+        if (!layer->disable_env.empty()) {
+            disable_value = !qgetenv(layer->disable_env.c_str()).isEmpty();
+
+            tooltip += layer->disable_env + format(": %s", disable_value ? "true" : "false");
+        }
+
+        if (!layer->enable_env.empty()) {
+            enable_value = qgetenv(layer->enable_env.c_str()).toStdString() == layer->enable_value;
+
+            tooltip += "; " + layer->enable_env + format(": %s", enable_value ? "true" : "false");
+        }
+
+        const std::string implicit_string = !disable_value || (!layer->enable_env.empty() && enable_value)
+                                                ? "Env Variables Controlled: On"
+                                                : "Env Variables Controlled: Off";
+
+        widget->setToolTip(tooltip.c_str());
+        widget->addItem(is_implicit_layer ? implicit_string.c_str() : "Application-Controlled");
+    } else {
+        widget->addItem("Application-Controlled");
+        widget->setCurrentIndex(LAYER_STATE_APPLICATION_CONTROLLED);
+    }
 
-    widget->addItem(is_implicit_layer ? "Implicitly On" : "Application-Controlled");
     widget->addItem("Overridden / Forced On");
     widget->addItem("Excluded / Forced Off");
     widget->setCurrentIndex(parameter.state);
 
-    connect(widget, SIGNAL(selectionMade(QTreeWidgetItem *, int)), this, SLOT(layerUseChanged(QTreeWidgetItem *, int)));
+    this->connect(widget, SIGNAL(selectionMade(QTreeWidgetItem *, int)), this, SLOT(layerUseChanged(QTreeWidgetItem *, int)));
 }
 
 void LayersDialog::LoadAvailableLayersUI() {
@@ -373,8 +389,8 @@ void LayersDialog::LoadUserDefinedPaths() {
         item->setExpanded(true);
 
         // Look for layers that are loaded that are also from this folder
-        for (std::size_t i = 0, n = configurator.layers.available_layers.size(); i < n; ++i) {
-            const Layer &layer = configurator.layers.available_layers[i];
+        for (std::size_t i = 0, n = configurator.layers.selected_layers.size(); i < n; ++i) {
+            const Layer &layer = configurator.layers.selected_layers[i];
 
             const QFileInfo file_info(layer.manifest_path.c_str());
             const std::string path(ConvertNativeSeparators(file_info.path().toStdString()));
@@ -407,29 +423,29 @@ void LayersDialog::resizeEvent(QResizeEvent *event) {
     (void)event;
 
     const QFontMetrics fm = ui->layerTree->fontMetrics();
-    const int combo_width = (fm.size(Qt::TextSingleLine, "Application-Controlled").width() * 1.6);
+    const int combo_width = (fm.size(Qt::TextSingleLine, "Env Variables Controlled: Off").width() * 1.5);
     const int width = ui->layerTree->width() - combo_width;
     ui->layerTree->setColumnWidth(0, width);
 }
 
 void LayersDialog::on_button_properties_clicked() {
-    const std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
-    const Layer *layer = FindByKey(available_layers, this->selected_available_layer_name.c_str());
+    const std::vector<Layer> &selected_layers = Configurator::Get().layers.selected_layers;
+    const Layer *layer = FindByKey(selected_layers, this->selected_available_layer_name.c_str());
 
     Alert::LayerProperties(layer);
 }
 
 void LayersDialog::on_button_doc_clicked() {
-    const std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
-    const Layer *layer = FindByKey(available_layers, this->selected_available_layer_name.c_str());
+    const std::vector<Layer> &selected_layers = Configurator::Get().layers.selected_layers;
+    const Layer *layer = FindByKey(selected_layers, this->selected_available_layer_name.c_str());
     const std::string path = format("%s/%s.html", GetPath(BUILTIN_PATH_APPDATA).c_str(), layer->key.c_str());
     ExportHtmlDoc(*layer, path);
     QDesktopServices::openUrl(QUrl(("file:///" + path).c_str()));
 }
 
 void LayersDialog::on_button_website_clicked() {
-    const std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
-    const Layer *layer = FindByKey(available_layers, this->selected_available_layer_name.c_str());
+    const std::vector<Layer> &selected_layers = Configurator::Get().layers.selected_layers;
+    const Layer *layer = FindByKey(selected_layers, this->selected_available_layer_name.c_str());
     QDesktopServices::openUrl(QUrl(layer->url.c_str()));
 }
 
@@ -459,7 +475,7 @@ void LayersDialog::on_button_reset_clicked() {
     selected_available_layer_name.clear();
     selected_sorted_layer_name.clear();
 
-    configuration.Reset(configurator.layers.available_layers, configurator.path);
+    configuration.Reset(configurator.layers.selected_layers, configurator.path);
 
     this->Reinit();
 
@@ -480,7 +496,7 @@ void LayersDialog::OverrideOrder(const std::string layer_name, const TreeWidgetI
 
     std::swap(below_parameter->overridden_rank, above_parameter->overridden_rank);
 
-    OrderParameter(configuration.parameters, Configurator::Get().layers.available_layers);
+    OrderParameter(configuration.parameters, Configurator::Get().layers.selected_layers);
     LoadAvailableLayersUI();
     LoadSortedLayersUI();
     LoadUserDefinedPaths();
@@ -536,7 +552,7 @@ void LayersDialog::UpdateButtons() {
     bool enabled_url = false;
 
     if (enabled) {
-        std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
+        std::vector<Layer> &available_layers = Configurator::Get().layers.selected_layers;
         const Layer *layer = FindByKey(available_layers, selected_available_layer_name.c_str());
         assert(layer != nullptr);
         if (!layer->url.empty()) enabled_url = true;
@@ -552,7 +568,7 @@ void LayersDialog::OverrideAllExplicitLayers() {
         if (it->state != LAYER_STATE_APPLICATION_CONTROLLED) continue;
 
         Configurator &configurator = Configurator::Get();
-        std::vector<Layer> &available_layers = configurator.layers.available_layers;
+        std::vector<Layer> &available_layers = configurator.layers.selected_layers;
 
         const Layer *layer = FindByKey(available_layers, it->key.c_str());
         if (layer == nullptr) continue;
@@ -585,13 +601,13 @@ void LayersDialog::layerUseChanged(QTreeWidgetItem *item, int selection) {
             OverrideAllExplicitLayers();
         }
     } else if (layer_state == LAYER_STATE_EXCLUDED) {
-        const std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
-        const Layer *layer = FindByKey(available_layers, tree_layer_item->layer_name.c_str());
+        const std::vector<Layer> &selected_layers = Configurator::Get().layers.selected_layers;
+        const Layer *layer = FindByKey(selected_layers, tree_layer_item->layer_name.c_str());
 
         if (layer != nullptr) {
             if (layer->type == LAYER_TYPE_IMPLICIT) {
                 if (Alert::LayerImplicitExcluded(tree_layer_item->layer_name.c_str()) == QMessageBox::No) {
-                    current_parameter->state = LAYER_STATE_APPLICATION_CONTROLLED;
+                    layer_state = LAYER_STATE_APPLICATION_CONTROLLED;
                 }
             }
         }
@@ -600,7 +616,7 @@ void LayersDialog::layerUseChanged(QTreeWidgetItem *item, int selection) {
     current_parameter->state = layer_state;
     current_parameter->overridden_rank = Parameter::NO_RANK;
 
-    OrderParameter(this->configuration.parameters, Configurator::Get().layers.available_layers);
+    OrderParameter(this->configuration.parameters, Configurator::Get().layers.selected_layers);
 
     ui->button_reset->setEnabled(true);
 
@@ -661,6 +677,13 @@ void LayersDialog::accept() {
         return;
     }
 
+    /* ANSI character are allowed
+    if (ui->lineEditName->text().contains(QRegularExpression(QStringLiteral("[^\\x{0000}-\\x{007F}]")))) {
+        Alert::ConfigurationNameASCII();
+        return;
+    }
+    */
+
     Configurator &configurator = Configurator::Get();
     if (this->configuration.key != ui->lineEditName->text().toStdString() &&
         IsFound(configurator.configurations.available_configurations, ui->lineEditName->text().toStdString().c_str())) {
@@ -671,7 +694,7 @@ void LayersDialog::accept() {
     Version loader_version;
     if (!configurator.SupportDifferentLayerVersions(&loader_version)) {
         std::string log_versions;
-        if (!configurator.configurations.CheckLayersVersions(configurator.layers.available_layers, &this->configuration,
+        if (!configurator.configurations.CheckLayersVersions(configurator.layers.selected_layers, &this->configuration,
                                                              log_versions)) {
             Alert::LayerIncompatibleVersions(log_versions.c_str(), loader_version);
             return;
@@ -688,16 +711,15 @@ void LayersDialog::accept() {
         configurator.configurations.RemoveConfigurationFile(saved_configuration->key);
     }
 
-    const std::string active_configuration_name = saved_configuration->key = ui->lineEditName->text().toStdString();
+    const std::string active_configuration_name = ui->lineEditName->text().toStdString();
+
+    saved_configuration->key = active_configuration_name;
     saved_configuration->description = ui->lineEditDescription->text().toStdString();
     saved_configuration->parameters = this->configuration.parameters;
     saved_configuration->user_defined_paths = this->configuration.user_defined_paths;
     saved_configuration->setting_tree_state.clear();
+    configurator.configurations.SaveAllConfigurations(configurator.layers.selected_layers);
 
-    configurator.configurations.SaveAllConfigurations(configurator.layers.available_layers);
-    configurator.configurations.LoadAllConfigurations(configurator.layers.available_layers);
-
-    configurator.configurations.SetActiveConfiguration(configurator.layers.available_layers, active_configuration_name.c_str());
     QDialog::accept();
 }
 
@@ -718,5 +740,5 @@ void LayersDialog::reject() {
 
 void LayersDialog::BuildParameters() {
     Configurator &configurator = Configurator::Get();
-    this->configuration.parameters = GatherParameters(this->configuration.parameters, configurator.layers.available_layers);
+    this->configuration.parameters = GatherParameters(this->configuration.parameters, configurator.layers.selected_layers);
 }
