@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2020-2021 Valve Corporation
- * Copyright (c) 2020-2021 LunarG, Inc.
+ * Copyright (c) 2020-2024 Valve Corporation
+ * Copyright (c) 2020-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -50,7 +50,7 @@
 static const char *TOOLTIP_ORDER =
     "Layers are executed between the Vulkan application and driver in the specific order represented here";
 
-SettingsTreeManager::SettingsTreeManager() : tree(nullptr) {}
+SettingsTreeManager::SettingsTreeManager() : launched_application(false), tree(nullptr) {}
 
 void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
     assert(build_tree);
@@ -62,8 +62,10 @@ void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
 
     this->tree = build_tree;
 
-    Configuration *configuration = configurator.configurations.GetActiveConfiguration();
-    assert(configuration != nullptr);
+    Configuration *configuration = configurator.configurations.FindActiveConfiguration();
+    if (configuration == nullptr) {
+        return;
+    }
 
     this->tree->blockSignals(true);
     this->tree->clear();
@@ -100,8 +102,8 @@ void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
 
             if (parameter.state != LAYER_STATE_OVERRIDDEN) continue;
 
-            const std::vector<Layer> &available_layers = configurator.layers.available_layers;
-            const Layer *layer = FindByKey(available_layers, parameter.key.c_str());
+            const std::vector<Layer> &selected_layers = configurator.layers.selected_layers;
+            const Layer *layer = FindByKey(selected_layers, parameter.key.c_str());
 
             QTreeWidgetItem *layer_item = new QTreeWidgetItem();
             this->tree->addTopLevelItem(layer_item);
@@ -109,11 +111,11 @@ void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
             std::string layer_text = parameter.key;
             if (layer == nullptr) {
                 layer_text += " (Missing)";
-                layer_item->setDisabled(true);
             } else if (layer->status != STATUS_STABLE) {
                 layer_text += std::string(" (") + GetToken(layer->status) + ")";
             }
 
+            layer_item->setToolTip(0, parameter.key.c_str());  // Hack for the context menu to find the layer
             layer_item->setText(0, layer_text.c_str());
             layer_item->setFont(0, font_layer);
             layer_item->setSizeHint(0, QSize(0, ITEM_HEIGHT));
@@ -155,7 +157,7 @@ void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
         }
 
         const std::size_t excluded_layer_count =
-            CountExcludedLayers(configuration->parameters, configurator.layers.available_layers);
+            CountExcludedLayers(configuration->parameters, configurator.layers.selected_layers);
 
         if (excluded_layer_count > 0) {
             // The last item is just the excluded layers
@@ -173,7 +175,7 @@ void SettingsTreeManager::CreateGUI(QTreeWidget *build_tree) {
 
                 if (parameter.state != LAYER_STATE_EXCLUDED) continue;
 
-                const Layer *layer = FindByKey(configurator.layers.available_layers, parameter.key.c_str());
+                const Layer *layer = FindByKey(configurator.layers.selected_layers, parameter.key.c_str());
                 if (layer == nullptr) continue;  // Do not display missing excluded layers
 
                 QTreeWidgetItem *layer_item = new QTreeWidgetItem();
@@ -210,11 +212,11 @@ void SettingsTreeManager::CleanupGUI() {
 
     Configurator &configurator = Configurator::Get();
 
-    Configuration *configuration = configurator.configurations.GetActiveConfiguration();
-    if (configuration == nullptr) return;
-
-    configuration->setting_tree_state.clear();
-    GetTreeState(configuration->setting_tree_state, this->tree->invisibleRootItem());
+    Configuration *configuration = configurator.configurations.FindActiveConfiguration();
+    if (configuration != nullptr) {
+        configuration->setting_tree_state.clear();
+        GetTreeState(configuration->setting_tree_state, this->tree->invisibleRootItem());
+    }
 
     this->validation.reset();
 
@@ -230,7 +232,7 @@ void SettingsTreeManager::OnExpandedChanged(const QModelIndex &index) {
 
     Configurator &configurator = Configurator::Get();
 
-    Configuration *configuration = configurator.configurations.GetActiveConfiguration();
+    Configuration *configuration = configurator.configurations.FindActiveConfiguration();
     configuration->setting_tree_state.clear();
     GetTreeState(configuration->setting_tree_state, this->tree->invisibleRootItem());
 
@@ -245,7 +247,7 @@ void SettingsTreeManager::OnCollapsedChanged(const QModelIndex &index) {
 
     Configurator &configurator = Configurator::Get();
 
-    Configuration *configuration = configurator.configurations.GetActiveConfiguration();
+    Configuration *configuration = configurator.configurations.FindActiveConfiguration();
     configuration->setting_tree_state.clear();
     GetTreeState(configuration->setting_tree_state, this->tree->invisibleRootItem());
 
@@ -254,8 +256,8 @@ void SettingsTreeManager::OnCollapsedChanged(const QModelIndex &index) {
 
 void SettingsTreeManager::BuildValidationTree(QTreeWidgetItem *parent, Parameter &parameter) {
     Configurator &configurator = Configurator::Get();
-    std::vector<Layer> &available_layers = configurator.layers.available_layers;
-    Layer *validation_layer = FindByKey(available_layers, "VK_LAYER_KHRONOS_validation");
+    std::vector<Layer> &selected_layers = configurator.layers.selected_layers;
+    Layer *validation_layer = FindByKey(selected_layers, "VK_LAYER_KHRONOS_validation");
     assert(validation_layer != nullptr);
 
     QTreeWidgetItem *validation_areas_item = new QTreeWidgetItem();
@@ -298,7 +300,7 @@ void SettingsTreeManager::BuildTreeItem(QTreeWidgetItem *parent, Parameter &para
     if (!IsPlatformSupported(meta_object.platform_flags)) return;
     if (meta_object.view == SETTING_VIEW_HIDDEN) return;
     if (meta_object.view == SETTING_VIEW_ADVANCED &&
-        !Configurator::Get().configurations.GetActiveConfiguration()->view_advanced_settings)
+        !Configurator::Get().configurations.FindActiveConfiguration()->view_advanced_settings)
         return;
 
     QTreeWidgetItem *item = new QTreeWidgetItem();
@@ -420,9 +422,9 @@ void SettingsTreeManager::BuildTreeItem(QTreeWidgetItem *parent, Parameter &para
 }
 
 void SettingsTreeManager::BuildGenericTree(QTreeWidgetItem *parent, Parameter &parameter) {
-    std::vector<Layer> &available_layers = Configurator::Get().layers.available_layers;
+    std::vector<Layer> &selected_layers = Configurator::Get().layers.selected_layers;
 
-    const SettingMetaSet &settings = FindByKey(available_layers, parameter.key.c_str())->settings;
+    const SettingMetaSet &settings = FindByKey(selected_layers, parameter.key.c_str())->settings;
     for (std::size_t i = 0, n = settings.size(); i < n; ++i) {
         this->BuildTreeItem(parent, parameter, *settings[i]);
     }
@@ -468,16 +470,18 @@ void SettingsTreeManager::Refresh(RefreshAreas refresh_areas) {
 
     this->tree->blockSignals(false);
 
-    QSettings settings;
-    if (!settings.value("vkconfig_restart", false).toBool()) {
-        settings.setValue("vkconfig_restart", true);
+    if (this->launched_application) {
+        QSettings settings;
+        if (!settings.value(VKCONFIG_KEY_MESSAGE_NEED_APPLICATION_RESTART, false).toBool()) {
+            settings.setValue(VKCONFIG_KEY_MESSAGE_NEED_APPLICATION_RESTART, true);
 
-        Alert::ConfiguratorRestart();
+            Alert::ConfiguratorRestart();
+        }
     }
 
     // Refresh layer configuration
     Configurator &configurator = Configurator::Get();
-    configurator.configurations.RefreshConfiguration(configurator.layers.available_layers);
+    configurator.configurations.Configure(configurator.layers.selected_layers);
 }
 
 void SettingsTreeManager::RefreshItem(RefreshAreas refresh_areas, QTreeWidgetItem *parent) {
