@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2020-2021 Valve Corporation
- * Copyright (c) 2020-2021 LunarG, Inc.
+ * Copyright (c) 2020-2024 Valve Corporation
+ * Copyright (c) 2020-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,33 +27,32 @@
 #include <QMessageBox>
 #include <QFileInfoList>
 
-static const char *SUPPORTED_CONFIG_FILES[] = {"_2_2_3", "_2_2_2", "_2_2_1"};
-
-ConfigurationManager::ConfigurationManager(Environment &environment) : active_configuration(nullptr), environment(environment) {}
+ConfigurationManager::ConfigurationManager(Environment &environment) : environment(environment) {}
 
 ConfigurationManager::~ConfigurationManager() {}
 
 void ConfigurationManager::LoadAllConfigurations(const std::vector<Layer> &available_layers) {
     this->available_configurations.clear();
-    this->active_configuration = nullptr;
 
     // If this is the first time, we need to create the initial set of
     // configuration files.
-    if (environment.first_run) {
-        RemoveConfigurationFiles();
-        LoadDefaultConfigurations(available_layers);
+    if (this->environment.first_run) {
+        this->RemoveConfigurationFiles();
+        this->LoadDefaultConfigurations(available_layers);
 
-        environment.first_run = false;
+        this->environment.first_run = false;
     }
 
     const std::string base_config_path = GetPath(BUILTIN_PATH_CONFIG_REF);
 
-    for (std::size_t i = 0, n = countof(SUPPORTED_CONFIG_FILES); i < n; ++i) {
+    const std::vector<std::string> &SUPPORTED_CONFIG_FILES = this->environment.paths.SUPPORTED_CONFIG_FILES;
+
+    for (std::size_t i = 0, n = SUPPORTED_CONFIG_FILES.size(); i < n; ++i) {
         const std::string path = base_config_path + SUPPORTED_CONFIG_FILES[i];
-        LoadConfigurationsPath(available_layers, path.c_str());
+        this->LoadConfigurationsPath(available_layers, path.c_str());
     }
 
-    RefreshConfiguration(available_layers);
+    this->SortConfigurations();
 }
 
 void ConfigurationManager::LoadDefaultConfigurations(const std::vector<Layer> &available_layers) {
@@ -64,7 +63,17 @@ void ConfigurationManager::LoadDefaultConfigurations(const std::vector<Layer> &a
         const bool result = configuration.Load(available_layers, configuration_files[i].absoluteFilePath().toStdString());
         assert(result);
 
-        if (!IsPlatformSupported(configuration.platform_flags)) continue;
+        if (!IsPlatformSupported(configuration.platform_flags)) {
+            continue;
+        }
+
+#ifdef _M_ARM64
+        if (configuration.key == "Frame Capture") {
+            continue;
+        } else if (configuration.key == "Crash Diagnostic") {
+            continue;
+        }
+#endif
 
         OrderParameter(configuration.parameters, available_layers);
 
@@ -78,8 +87,6 @@ void ConfigurationManager::LoadDefaultConfigurations(const std::vector<Layer> &a
 }
 
 void ConfigurationManager::SortConfigurations() {
-    this->active_configuration = nullptr;
-
     struct Compare {
         bool operator()(const Configuration &a, const Configuration &b) const {
             const bool result = a.key < b.key;
@@ -103,11 +110,19 @@ void ConfigurationManager::LoadConfigurationsPath(const std::vector<Layer> &avai
             path = ":/configurations/Portability.json";
         }
         const bool result = configuration.Load(available_layers, path);
-        if (!result) continue;
+        if (!result) {
+            continue;
+        }
 
-        if (FindByKey(available_configurations, configuration.key.c_str()) != nullptr) continue;
+        if (FindByKey(available_configurations, configuration.key.c_str()) != nullptr) {
+            continue;
+        }
+
+        std::string missing_layer;
+        if (!HasMissingLayer(configuration.parameters, available_layers, missing_layer)) {
+            OrderParameter(configuration.parameters, available_layers);
+        }
 
-        OrderParameter(configuration.parameters, available_layers);
         available_configurations.push_back(configuration);
     }
 }
@@ -134,8 +149,6 @@ Configuration &ConfigurationManager::CreateConfiguration(const std::vector<Layer
     const bool result = configuration.Load(available_layers, path.c_str());
     assert(result);
 
-    RemoveConfigurationFile(new_configuration.key);
-
     this->available_configurations.push_back(configuration);
     this->SortConfigurations();
 
@@ -145,7 +158,9 @@ Configuration &ConfigurationManager::CreateConfiguration(const std::vector<Layer
 bool ConfigurationManager::HasFile(const Configuration &configuration) const {
     const std::string base_path = GetPath(BUILTIN_PATH_CONFIG_REF);
 
-    for (std::size_t i = 0, n = countof(SUPPORTED_CONFIG_FILES); i < n; ++i) {
+    const std::vector<std::string> &SUPPORTED_CONFIG_FILES = environment.paths.SUPPORTED_CONFIG_FILES;
+
+    for (std::size_t i = 0, n = SUPPORTED_CONFIG_FILES.size(); i < n; ++i) {
         const std::string path = base_path + SUPPORTED_CONFIG_FILES[i] + "/" + configuration.key + ".json";
 
         std::FILE *file = std::fopen(path.c_str(), "r");
@@ -160,12 +175,14 @@ bool ConfigurationManager::HasFile(const Configuration &configuration) const {
 void ConfigurationManager::RemoveConfigurationFiles() {
     const std::string base_path = GetPath(BUILTIN_PATH_CONFIG_REF);
 
-    for (std::size_t i = 0, n = countof(SUPPORTED_CONFIG_FILES); i < n; ++i) {
+    const std::vector<std::string> &SUPPORTED_CONFIG_FILES = environment.paths.SUPPORTED_CONFIG_FILES;
+
+    for (std::size_t i = 0, n = SUPPORTED_CONFIG_FILES.size(); i < n; ++i) {
         const std::string path = base_path + SUPPORTED_CONFIG_FILES[i];
 
         const QFileInfoList &configuration_files = GetJSONFiles(path.c_str());
         for (int i = 0, n = configuration_files.size(); i < n; ++i) {
-            std::remove(configuration_files[i].filePath().toStdString().c_str());
+            QFile::remove(configuration_files[i].filePath());
         }
     }
 }
@@ -173,14 +190,16 @@ void ConfigurationManager::RemoveConfigurationFiles() {
 void ConfigurationManager::RemoveConfigurationFile(const std::string &key) {
     const std::string base_path = GetPath(BUILTIN_PATH_CONFIG_REF);
 
-    for (std::size_t i = 0, n = countof(SUPPORTED_CONFIG_FILES); i < n; ++i) {
+    const std::vector<std::string> &SUPPORTED_CONFIG_FILES = environment.paths.SUPPORTED_CONFIG_FILES;
+
+    for (std::size_t i = 0, n = SUPPORTED_CONFIG_FILES.size(); i < n; ++i) {
         const std::string path = base_path + SUPPORTED_CONFIG_FILES[i];
 
         const QFileInfoList &configuration_files = GetJSONFiles(path.c_str());
         for (int j = 0, o = configuration_files.size(); j < o; ++j) {
             const QString filename = configuration_files[j].fileName();
             if (filename.toStdString() == key + ".json") {
-                std::remove(configuration_files[j].filePath().toStdString().c_str());
+                QFile::remove(configuration_files[j].filePath());
             }
         }
     }
@@ -189,82 +208,113 @@ void ConfigurationManager::RemoveConfigurationFile(const std::string &key) {
 void ConfigurationManager::RemoveConfiguration(const std::vector<Layer> &available_layers, const std::string &configuration_name) {
     assert(!configuration_name.empty());
 
-    // Not the active configuration
-    if (active_configuration != nullptr) {
-        if (active_configuration->key == configuration_name.c_str()) {
-            SetActiveConfiguration(available_layers, nullptr);
-        }
-    }
+    LayersMode saved_mode = this->environment.GetMode();
+    this->environment.SetMode(LAYERS_MODE_BY_APPLICATIONS);
+    this->Configure(available_layers);
 
     RemoveConfigurationFile(configuration_name.c_str());
 
     // Update the configuration in the list
     std::vector<Configuration> updated_configurations;
 
-    for (std::size_t i = 0, n = available_configurations.size(); i < n; ++i) {
-        if (available_configurations[i].key == configuration_name) continue;
-        updated_configurations.push_back(available_configurations[i]);
-    }
-
-    std::swap(updated_configurations, available_configurations);
-
-    SetActiveConfiguration(available_layers, nullptr);
-}
-
-void ConfigurationManager::SetActiveConfiguration(const std::vector<Layer> &available_layers,
-                                                  const std::string &configuration_name) {
-    assert(!configuration_name.empty());
-
-    Configuration *configuration = FindByKey(available_configurations, configuration_name.c_str());
-    assert(configuration != nullptr);
-
-    SetActiveConfiguration(available_layers, configuration);
-}
-
-void ConfigurationManager::SetActiveConfiguration(const std::vector<Layer> &available_layers, Configuration *active_configuration) {
-    this->active_configuration = active_configuration;
-
-    bool surrender = false;
-    if (this->active_configuration != nullptr) {
-        assert(!this->active_configuration->key.empty());
-        environment.Set(ACTIVE_CONFIGURATION, this->active_configuration->key.c_str());
-        surrender = !this->active_configuration->HasOverride();
-    } else {
-        this->active_configuration = nullptr;
-        surrender = true;
-    }
-
-    if (surrender) {
-        SurrenderConfiguration(environment);
-    } else {
-        assert(this->active_configuration != nullptr);
-        OverrideConfiguration(environment, available_layers, *active_configuration);
-    }
-}
-
-void ConfigurationManager::RefreshConfiguration(const std::vector<Layer> &available_layers) {
-    const std::string active_configuration_name = environment.Get(ACTIVE_CONFIGURATION);
-
-    if (!active_configuration_name.empty() && environment.UseOverride()) {
-        Configuration *active_configuration = FindByKey(available_configurations, active_configuration_name.c_str());
-        if (active_configuration == nullptr) {
-            environment.Set(ACTIVE_CONFIGURATION, "");
+    for (std::size_t i = 0, n = this->available_configurations.size(); i < n; ++i) {
+        if (this->available_configurations[i].key == configuration_name) {
+            continue;
         }
-        SetActiveConfiguration(available_layers, active_configuration);
-    } else {
-        SetActiveConfiguration(available_layers, nullptr);
+        updated_configurations.push_back(this->available_configurations[i]);
     }
+
+    std::swap(updated_configurations, this->available_configurations);
+    this->SortConfigurations();
+
+    this->environment.SetSelectedConfiguration("");
+    this->environment.SetMode(saved_mode);
+    this->Configure(available_layers);
+}
+
+void ConfigurationManager::Configure(const std::vector<Layer> &available_layers) {
+    const std::string active_configuration_name = this->environment.GetSelectedConfiguration();
+
+    if (active_configuration_name.empty()) {
+        this->environment.SetSelectedConfiguration("");
+        this->Configure(available_layers, "");
+    } else {
+        Configuration *selected_configuration = FindByKey(this->available_configurations, active_configuration_name.c_str());
+        if (selected_configuration == nullptr) {
+            environment.SetSelectedConfiguration("");
+        }
+        this->Configure(available_layers, this->environment.GetSelectedConfiguration());
+    }
+}
+
+void ConfigurationManager::Configure(const std::vector<Layer> &available_layers, const std::string &configuration_name) {
+    switch (this->environment.GetMode()) {
+        default:
+        case LAYERS_MODE_BY_APPLICATIONS: {
+            SurrenderConfiguration(this->environment);
+        } break;
+        case LAYERS_MODE_BY_CONFIGURATOR_RUNNING: {
+            if (configuration_name.empty()) {
+                ::SurrenderConfiguration(this->environment);
+            } else {
+                Configuration *selected_configuration = FindByKey(this->available_configurations, configuration_name.c_str());
+                std::string missing_layer;
+                if (::HasMissingLayer(selected_configuration->parameters, available_layers, missing_layer)) {
+                    ::SurrenderConfiguration(this->environment);
+                } else {
+                    ::OverrideConfiguration(this->environment, available_layers, *selected_configuration);
+                }
+            }
+        } break;
+        case LAYERS_MODE_BY_CONFIGURATOR_ALL_DISABLED: {
+            Configuration temp_configuration;
+            temp_configuration.key = "_TempConfiguration";
+            temp_configuration.parameters = GatherParameters(temp_configuration.parameters, available_layers);
+
+            for (std::size_t i = 0, n = temp_configuration.parameters.size(); i < n; ++i) {
+                temp_configuration.parameters[i].state = LAYER_STATE_EXCLUDED;
+            }
+
+            ::OverrideConfiguration(environment, available_layers, temp_configuration);
+        } break;
+    }
+}
+
+Configuration *ConfigurationManager::FindActiveConfiguration() {
+    if (this->environment.GetSelectedConfiguration().empty()) {
+        return nullptr;
+    }
+
+    return FindByKey(this->available_configurations, this->environment.GetSelectedConfiguration().c_str());
 }
 
 bool ConfigurationManager::HasActiveConfiguration(const std::vector<Layer> &available_layers) const {
-    std::string missing_layer;
-    if (this->active_configuration != nullptr)
-        return !HasMissingLayer(this->active_configuration->parameters, available_layers, missing_layer);
-    else
-        return false;
+    switch (environment.GetMode()) {
+        case LAYERS_MODE_BY_APPLICATIONS:
+            return false;
+        case LAYERS_MODE_BY_CONFIGURATOR_RUNNING: {
+            const std::string configuration_name = this->environment.GetSelectedConfiguration();
+            if (configuration_name.empty()) {
+                return false;
+            } else {
+                const Configuration *selected_configuration = FindByKey(this->available_configurations, configuration_name.c_str());
+                if (selected_configuration != nullptr) {
+                    std::string missing_layer;
+                    return !::HasMissingLayer(selected_configuration->parameters, available_layers, missing_layer);
+                } else {
+                    return false;
+                }
+            }
+        }
+        case LAYERS_MODE_BY_CONFIGURATOR_ALL_DISABLED:
+            return true;
+    }
+
+    return false;
 }
 
-void ConfigurationManager::ImportConfiguration(const std::vector<Layer> &available_layers, const std::string &full_import_path) {
+std::string ConfigurationManager::ImportConfiguration(const std::vector<Layer> &available_layers,
+                                                      const std::string &full_import_path) {
     assert(!full_import_path.empty());
 
     Configuration configuration;
@@ -275,13 +325,14 @@ void ConfigurationManager::ImportConfiguration(const std::vector<Layer> &availab
         msg.setText("Cannot access the source configuration file.");
         msg.setInformativeText(full_import_path.c_str());
         msg.exec();
-        return;
+        return "";
     }
 
     configuration.key = MakeConfigurationName(this->available_configurations, configuration.key + " (Imported)");
     this->available_configurations.push_back(configuration);
     this->SortConfigurations();
-    SetActiveConfiguration(available_layers, configuration.key.c_str());
+
+    return configuration.key;
 }
 
 void ConfigurationManager::ExportConfiguration(const std::vector<Layer> &available_layers, const std::string &full_export_path,
@@ -303,23 +354,18 @@ void ConfigurationManager::ExportConfiguration(const std::vector<Layer> &availab
 }
 
 void ConfigurationManager::ResetDefaultsConfigurations(const std::vector<Layer> &available_layers) {
-    // Clear the current configuration as we may be about to remove it.
-    SetActiveConfiguration(available_layers, nullptr);
-
-    environment.Reset(Environment::DEFAULT);
+    this->environment.Reset(Environment::DEFAULT);
 
     // Now we need to kind of restart everything
-    LoadAllConfigurations(available_layers);
+    this->LoadAllConfigurations(available_layers);
+    this->SaveAllConfigurations(available_layers);
 }
 
 void ConfigurationManager::ReloadDefaultsConfigurations(const std::vector<Layer> &available_layers) {
-    // Clear the current configuration as we may be about to remove it.
-    SetActiveConfiguration(available_layers, nullptr);
-
     // Now we need to kind of restart everything
     LoadDefaultConfigurations(available_layers);
 
-    RefreshConfiguration(available_layers);
+    this->Configure(available_layers);
 }
 
 void ConfigurationManager::FirstDefaultsConfigurations(const std::vector<Layer> &available_layers) {
@@ -347,31 +393,29 @@ void ConfigurationManager::FirstDefaultsConfigurations(const std::vector<Layer>
         OrderParameter(configuration.parameters, available_layers);
         available_configurations.push_back(configuration);
     }
-
-    RefreshConfiguration(available_layers);
 }
 
-bool ConfigurationManager::CheckApiVersions(const std::vector<Layer> &available_layers, Configuration *active_configuration,
+bool ConfigurationManager::CheckApiVersions(const std::vector<Layer> &available_layers, Configuration *selected_configuration,
                                             std::string &log_versions) const {
-    return CompareLayersVersions(available_layers, active_configuration, environment.api_version, log_versions, true);
+    return CompareLayersVersions(available_layers, selected_configuration, environment.api_version, log_versions, true);
 }
 
-bool ConfigurationManager::CheckLayersVersions(const std::vector<Layer> &available_layers, Configuration *active_configuration,
+bool ConfigurationManager::CheckLayersVersions(const std::vector<Layer> &available_layers, Configuration *selected_configuration,
                                                std::string &log_versions) const {
-    return CompareLayersVersions(available_layers, active_configuration, Version::VERSION_NULL, log_versions, false);
+    return CompareLayersVersions(available_layers, selected_configuration, Version::VERSION_NULL, log_versions, false);
 }
 
-bool ConfigurationManager::CompareLayersVersions(const std::vector<Layer> &available_layers, Configuration *active_configuration,
+bool ConfigurationManager::CompareLayersVersions(const std::vector<Layer> &available_layers, Configuration *selected_configuration,
                                                  const Version &version, std::string &log_versions, bool is_less) const {
-    assert(active_configuration != nullptr);
+    assert(selected_configuration != nullptr);
 
     Version current_version = version;
 
     bool result = true;
 
-    for (std::size_t param_index = 0, param_count = active_configuration->parameters.size(); param_index < param_count;
+    for (std::size_t param_index = 0, param_count = selected_configuration->parameters.size(); param_index < param_count;
          ++param_index) {
-        const Parameter &parameter = active_configuration->parameters[param_index];
+        const Parameter &parameter = selected_configuration->parameters[param_index];
 
         if (parameter.state != LAYER_STATE_OVERRIDDEN) continue;
 
