@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2020-2021 Valve Corporation
- * Copyright (c) 2020-2021 LunarG, Inc.
+ * Copyright (c) 2020-2024 Valve Corporation
+ * Copyright (c) 2020-2024 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,14 +36,6 @@
 
 #include <cassert>
 
-// Saved settings for the application
-#define VKCONFIG_KEY_INITIALIZE_FILES "FirstTimeRun"
-#define VKCONFIG_KEY_OVERRIDE_MODE "OverrideMode"
-#define VKCONFIG_KEY_LOADER_MESSAGE "LoaderMessage"
-
-#define VKCONFIG_KEY_VKCONFIG_VERSION "vkConfigVersion"
-#define VKCONFIG_KEY_CUSTOM_PATHS "customPaths"
-
 static const char* GetApplicationSuffix() {
     static const char* TABLE[] = {
         ".exe",  // PLATFORM_WINDOWS
@@ -56,79 +48,88 @@ static const char* GetApplicationSuffix() {
     return TABLE[VKC_PLATFORM];
 }
 
-static const char* GetActiveToken(Active active) {
-    assert(active >= ACTIVE_FIRST && active <= ACTIVE_LAST);
-
-    static const char* table[] = {
-        "activeProfile",  // ACTIVE_CONFIGURATION
-        "launchApp"       // ACTIVE_EXECUTABLE
-    };
-    static_assert(countof(table) == ACTIVE_COUNT, "The tranlation table size doesn't match the enum number of elements");
-
-    return table[active];
-}
-
-static const char* GetActiveDefault(Active active) {
-    assert(active >= ACTIVE_FIRST && active <= ACTIVE_LAST);
-
-    static const char* table[] = {
-        "Validation",  // ACTIVE_CONFIGURATION
-        ""             // ACTIVE_EXECUTABLE
-    };
-    static_assert(countof(table) == ACTIVE_COUNT, "The tranlation table size doesn't match the enum number of elements");
-
-    return table[active];
-}
-
 static const char* GetLayoutStateToken(LayoutState state) {
     assert(state >= LAYOUT_FIRST && state <= LAYOUT_LAST);
 
     static const char* table[] = {
-        "geometry",            // LAYOUT_GEOMETRY
-        "windowState",         // LAYOUT_WINDOW_STATE
-        "splitter1State",      // LAYOUT_MAIN_SPLITTER1
-        "splitter2State",      // LAYOUT_MAIN_SPLITTER2
-        "splitter3State",      // LAYOUT_MAIN_SPLITTER3
-        "layerGeometry",       // LAYOUT_GEOMETRY_SPLITTER
-        "splitterLayerState",  // LAYOUT_LAYER_SPLITTER
-        "launcherCollapsed",   // LAYOUT_LAUNCHER_COLLAPSED
-        "launcherOnClear"      // LAYOUT_LAUNCHER_CLEAR_ON
+        "geometry",                    // LAYOUT_GEOMETRY
+        "windowState",                 // LAYOUT_WINDOW_STATE
+        "splitter1State",              // LAYOUT_MAIN_SPLITTER1
+        "splitter2State",              // LAYOUT_MAIN_SPLITTER2
+        "splitter3State",              // LAYOUT_MAIN_SPLITTER3
+        "vkconfig2_6_geometry",        // VKCONFIG3_LAYOUT_GEOMETRY
+        "vkconfig2_6_windowState",     // VKCONFIG3_LAYOUT_WINDOW_STATE
+        "vkconfig2_6_splitter1State",  // VKCONFIG3_LAYOUT_MAIN_SPLITTER1
+        "vkconfig2_6_splitter2State",  // VKCONFIG3_LAYOUT_MAIN_SPLITTER2
+        "vkconfig2_6_splitter3State",  // VKCONFIG3_LAYOUT_MAIN_SPLITTER3
+        "layerGeometry",               // LAYOUT_GEOMETRY_SPLITTER
+        "splitterLayerState",          // LAYOUT_LAYER_SPLITTER
+        "launcherCollapsed",           // LAYOUT_LAUNCHER_COLLAPSED
+        "launcherOnClear"              // LAYOUT_LAUNCHER_CLEAR_ON
     };
     static_assert(countof(table) == LAYOUT_COUNT, "The tranlation table size doesn't match the enum number of elements");
 
     return table[state];
 }
 
-LoaderMessageLevel GetLoaderDebug(const std::string& value) {
+std::string GetLoaderMessageToken(LoaderMessageType message_type) {
+    static const char* LOADER_MESSAGE_TOKENS[]{
+        "error",  // LOADER_MESSAGE_ERROR
+        "warn",   // LOADER_MESSAGE_WARN
+        "info",   // LOADER_MESSAGE_INFO
+        "debug",  // LOADER_MESSAGE_DEBUG
+        "layer",  // LOADER_MESSAGE_LAYER
+        "implem"  // LOADER_MESSAGE_IMPLEMENTATION
+    };
+
+    static_assert(countof(LOADER_MESSAGE_TOKENS) == LOADER_MESSAGE_COUNT,
+                  "The tranlation table size doesn't match the enum number of elements");
+
+    return LOADER_MESSAGE_TOKENS[message_type - LOADER_MESSAGE_FIRST];
+}
+
+std::string GetLoaderMessageTokens(int mesage_types) {
+    std::vector<std::string> results;
+
+    for (int i = LOADER_MESSAGE_FIRST, l = LOADER_MESSAGE_LAST; i <= l; ++i) {
+        if (mesage_types & (1 << i)) {
+            results.push_back(GetLoaderMessageToken(static_cast<LoaderMessageType>(i)));
+        }
+    }
+
+    return Merge(results, ",");
+}
+
+int GetLoaderMessageTypes(const std::string& values) {
+    std::vector<std::string> split_values = Split(values, ",");
+
+    int result = 0;
+
+    for (std::size_t i = 0, n = split_values.size(); i < n; ++i) {
+        const LoaderMessageType message_type = GetLoaderMessageType(split_values[i]);
+        if (message_type == LOADER_MESSAGE_NONE) {
+            continue;
+        }
+        result |= (1 << message_type);
+    }
+
+    return result;
+}
+
+LoaderMessageType GetLoaderMessageType(const std::string& value) {
     for (int i = LOADER_MESSAGE_FIRST, n = LOADER_MESSAGE_LAST; i <= n; ++i) {
-        const LoaderMessageLevel level = static_cast<LoaderMessageLevel>(i);
-        if (GetLoaderDebugToken(level) == value) return level;
+        const LoaderMessageType message_type = static_cast<LoaderMessageType>(i);
+        if (::GetLoaderMessageToken(message_type) == value) return message_type;
     }
 
     return LOADER_MESSAGE_NONE;
 }
 
-std::string GetLoaderDebugToken(LoaderMessageLevel level) {
-    static const char* LOADER_MESSAGE_LEVEL[]{
-        "",        // LOADER_MESSAGE_NONE
-        "error",   // LOADER_MESSAGE_ERROR
-        "warn",    // LOADER_MESSAGE_WARN
-        "info",    // LOADER_MESSAGE_INFO
-        "debug",   // LOADER_MESSAGE_DEBUG
-        "layer",   // LOADER_MESSAGE_LAYER
-        "implem",  // LOADER_MESSAGE_IMPLEMENTATION
-        "all"      // LOADER_MESSAGE_ALL
-    };
-
-    static_assert(countof(LOADER_MESSAGE_LEVEL) == LOADER_MESSAGE_COUNT,
-                  "The tranlation table size doesn't match the enum number of elements");
-
-    return LOADER_MESSAGE_LEVEL[level];
-}
-
 Environment::Environment(PathManager& paths, const Version& api_version)
     : api_version(api_version),
-      loader_message_level(GetLoaderDebug(qgetenv("VK_LOADER_DEBUG").toStdString())),
+      use_system_tray(false),
+      use_per_application_configuration(false),
+      loader_message_types(::GetLoaderMessageTypes(qgetenv("VK_LOADER_DEBUG").toStdString())),
       paths_manager(paths),
       paths(paths_manager) {
     const bool result = Load();
@@ -143,17 +144,18 @@ Environment::~Environment() {
 void Environment::Reset(ResetMode mode) {
     switch (mode) {
         case DEFAULT: {
-            first_run = true;
-            version = Version::LAYER_CONFIG;
-            override_state = OVERRIDE_STATE_GLOBAL_TEMPORARY;
-
-            for (std::size_t i = 0; i < ACTIVE_COUNT; ++i) {
-                actives[i] = GetActiveDefault(static_cast<Active>(i));
-            }
+            this->first_run = true;
+            this->vkconfig2_version = Version::VKCONFIG;
+            this->vkconfig3_version = Version::VKCONFIG3;
+            this->layers_mode = LAYERS_MODE_BY_CONFIGURATOR_RUNNING;
+            this->use_per_application_configuration = false;
+            this->use_application_list = false;
+            this->use_system_tray = false;
+            this->selected_configuration = "Validation";
+            this->active_application.clear();
+            this->user_defined_layers_paths[USER_DEFINED_LAYERS_PATHS_GUI].clear();
 
             applications = CreateDefaultApplications();
-
-            Set(ACTIVE_CONFIGURATION, "Validation");
             break;
         }
         case CLEAR: {
@@ -166,8 +168,9 @@ void Environment::Reset(ResetMode mode) {
             settings.setValue("VKCONFIG_WARN_MISSING_LAYERS_IGNORE", false);
             settings.setValue("VKCONFIG_WARN_CORE_SHADER_IGNORE", false);
 
+            settings.setValue(VKCONFIG_KEY_MESSAGE_NEED_APPLICATION_RESTART, false);
             settings.setValue("overrideActive", false);
-            settings.setValue("applyOnlyToList", false);
+            settings.setValue("applyPerApplication", false);
             settings.setValue("keepActiveOnExit", false);
 
             settings.setValue("restartWarning", false);
@@ -175,11 +178,11 @@ void Environment::Reset(ResetMode mode) {
 
             const std::string loader_debug_message(qgetenv("VK_LOADER_DEBUG"));
             if (loader_debug_message.empty()) {
-                loader_message_level = LOADER_MESSAGE_NONE;
+                this->loader_message_types = 0;
             } else {
-                loader_message_level = GetLoaderDebug(loader_debug_message);
+                this->loader_message_types = ::GetLoaderMessageTypes(loader_debug_message);
             }
-            settings.setValue(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(loader_message_level));
+            settings.setValue(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(this->loader_message_types));
             break;
         }
         case SYSTEM: {
@@ -205,24 +208,35 @@ bool Environment::Load() {
     this->first_run = settings.value(VKCONFIG_KEY_INITIALIZE_FILES, first_run).toBool();
 
     // Load "version": If the version doesn't exist of it's an old version of vkconfig
-    this->version =
+    this->vkconfig2_version =
         Version(settings.value(VKCONFIG_KEY_VKCONFIG_VERSION, Version::VKCONFIG.str().c_str()).toString().toStdString());
 
+    // Load "version": If the version doesn't exist of it's an old version of vkconfig
+    this->vkconfig3_version =
+        Version(settings.value(VKCONFIG_KEY_VKCONFIG3_VERSION, Version::VKCONFIG3.str().c_str()).toString().toStdString());
+
     // Load 'override_mode"
-    override_state = static_cast<OverrideState>(settings.value(VKCONFIG_KEY_OVERRIDE_MODE, QVariant(override_state)).toInt());
+    this->layers_mode = static_cast<LayersMode>(settings.value(VKCONFIG_KEY_LAYERS_MODE, QVariant(layers_mode)).toInt());
+
+    this->use_per_application_configuration =
+        static_cast<LayersMode>(settings.value(VKCONFIG_KEY_STATE_PER_APPLICATION, this->use_application_list).toBool());
+    this->use_application_list =
+        static_cast<LayersMode>(settings.value(VKCONFIG_KEY_STATE_APPLICATION_LIST, this->use_application_list).toBool());
+    this->use_system_tray = static_cast<LayersMode>(settings.value(VKCONFIG_KEY_STATE_SYSTEM_TRAY, this->use_system_tray).toBool());
 
     // Load loader debug message state
-    loader_message_level = static_cast<LoaderMessageLevel>(
-        settings.value(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(loader_message_level)).toInt());
+    this->loader_message_types = settings.value(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(this->loader_message_types)).toInt();
 
     // Load active configuration
-    for (std::size_t i = 0; i < ACTIVE_COUNT; ++i) {
-        actives[i] = settings.value(GetActiveToken(static_cast<Active>(i)), actives[i].c_str()).toString().toStdString();
-    }
+    this->selected_configuration =
+        settings.value(VKCONFIG_KEY_ACTIVE_CONFIGURATION, this->selected_configuration.c_str()).toString().toStdString();
+
+    this->active_application =
+        settings.value(VKCONFIG_KEY_ACTIVE_APPLICATION, this->active_application.c_str()).toString().toStdString();
 
     // Load layout state
     for (std::size_t i = 0; i < LAYOUT_COUNT; ++i) {
-        layout_states[i] = settings.value(GetLayoutStateToken(static_cast<LayoutState>(i))).toByteArray();
+        this->layout_states[i] = settings.value(GetLayoutStateToken(static_cast<LayoutState>(i))).toByteArray();
     }
 
     // Load default configuration already init
@@ -282,7 +296,8 @@ bool Environment::LoadApplications() {
                     application.app_name = app_object.value("app_name").toString().toStdString();
                     application.executable_path = app_object.value("app_path").toString().toStdString();
                     application.working_folder = app_object.value("app_folder").toString().toStdString();
-                    application.override_layers = !app_object.value("exclude_override").toBool();
+                    application.layers_mode = app_object.value("exclude_override").toBool() ? LAYERS_MODE_BY_APPLICATIONS
+                                                                                            : LAYERS_MODE_BY_CONFIGURATOR_RUNNING;
                     application.log_file = app_object.value("log_file").toString().toStdString();
                     if (application.app_name.length() == 0) {
                         std::string path = application.executable_path.c_str();
@@ -314,25 +329,28 @@ bool Environment::Save() const {
     QSettings settings;
 
     // Save 'first_run'
-    settings.setValue(VKCONFIG_KEY_INITIALIZE_FILES, first_run);
+    settings.setValue(VKCONFIG_KEY_INITIALIZE_FILES, this->first_run);
 
     // Save 'version'
     settings.setValue(VKCONFIG_KEY_VKCONFIG_VERSION, Version::LAYER_CONFIG.str().c_str());
 
-    // Save 'override_mode'
-    settings.setValue(VKCONFIG_KEY_OVERRIDE_MODE, override_state);
+    settings.setValue(VKCONFIG_KEY_LAYERS_MODE, this->layers_mode);
+    settings.setValue(VKCONFIG_KEY_STATE_PER_APPLICATION, this->use_per_application_configuration);
+    settings.setValue(VKCONFIG_KEY_STATE_APPLICATION_LIST, this->use_application_list);
+    settings.setValue(VKCONFIG_KEY_STATE_SYSTEM_TRAY, this->use_system_tray);
 
     // Save 'loader_message'
-    settings.setValue(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(loader_message_level));
+    settings.setValue(VKCONFIG_KEY_LOADER_MESSAGE, static_cast<int>(this->loader_message_types));
 
-    // Save active state
-    for (std::size_t i = 0; i < ACTIVE_COUNT; ++i) {
-        settings.setValue(GetActiveToken(static_cast<Active>(i)), actives[i].c_str());
-    }
+    // Save active configuration
+    settings.setValue(VKCONFIG_KEY_ACTIVE_CONFIGURATION, this->selected_configuration.c_str());
+
+    // Save active application
+    settings.setValue(VKCONFIG_KEY_ACTIVE_APPLICATION, this->active_application.c_str());
 
     // Save layout state
     for (std::size_t i = 0; i < LAYOUT_COUNT; ++i) {
-        settings.setValue(GetLayoutStateToken(static_cast<LayoutState>(i)), layout_states[i]);
+        settings.setValue(GetLayoutStateToken(static_cast<LayoutState>(i)), this->layout_states[i]);
     }
 
     // Save default configuration initizalized
@@ -353,7 +371,7 @@ bool Environment::SaveApplications() const {
         application_object.insert("app_name", applications[i].app_name.c_str());
         application_object.insert("app_path", applications[i].executable_path.c_str());
         application_object.insert("app_folder", applications[i].working_folder.c_str());
-        application_object.insert("exclude_override", !applications[i].override_layers);
+        application_object.insert("exclude_override", applications[i].layers_mode == LAYERS_MODE_BY_APPLICATIONS);
         application_object.insert("log_file", applications[i].log_file.c_str());
 
         // Ground work for mulitiple sets of command line arguments
@@ -378,14 +396,14 @@ bool Environment::SaveApplications() const {
 }
 
 void Environment::SelectActiveApplication(std::size_t application_index) {
-    assert(application_index < applications.size());
+    assert(application_index < this->applications.size());
 
-    Set(ACTIVE_APPLICATION, applications[application_index].app_name.c_str());
+    this->SetActiveApplication(this->applications[application_index].app_name);
 }
 
 int Environment::GetActiveApplicationIndex() const {
-    for (std::size_t i = 0, n = applications.size(); i < n; ++i) {
-        if (applications[i].app_name.c_str() == Get(ACTIVE_APPLICATION)) {
+    for (std::size_t i = 0, n = this->applications.size(); i < n; ++i) {
+        if (this->applications[i].app_name == this->active_application) {
             return static_cast<int>(i);
         }
     }
@@ -394,112 +412,94 @@ int Environment::GetActiveApplicationIndex() const {
 }
 
 bool Environment::HasOverriddenApplications() const {
-    for (std::size_t i = 0, n = applications.size(); i < n; ++i) {
-        if (applications[i].override_layers) return true;
+    for (std::size_t i = 0, n = this->applications.size(); i < n; ++i) {
+        if (this->applications[i].layers_mode != LAYERS_MODE_BY_APPLICATIONS) {
+            return true;
+        }
     }
 
     return false;
 }
 
 bool Environment::AppendApplication(const Application& application) {
-    applications.push_back(application);
+    this->applications.push_back(application);
     return true;
 }
 
 bool Environment::RemoveApplication(std::size_t application_index) {
-    assert(!applications.empty());
-    assert(application_index < applications.size());
+    assert(!this->applications.empty());
+    assert(application_index < this->applications.size());
 
-    if (applications.size() == 1u) {
-        applications.clear();
+    if (this->applications.size() == 1u) {
+        this->applications.clear();
         return true;
     }
 
     std::vector<Application> new_applications;
-    new_applications.reserve(applications.size() - 1);
+    new_applications.reserve(this->applications.size() - 1);
 
-    for (std::size_t i = 0, n = applications.size(); i < n; ++i) {
+    for (std::size_t i = 0, n = this->applications.size(); i < n; ++i) {
         if (i == application_index) continue;
-        new_applications.push_back(applications[i]);
+        new_applications.push_back(this->applications[i]);
     }
 
-    std::swap(applications, new_applications);
+    std::swap(this->applications, new_applications);
     return true;
 }
 
 const Application& Environment::GetActiveApplication() const {
-    assert(!applications.empty());
+    assert(!this->applications.empty());
 
-    for (std::size_t i = 0, n = applications.size(); i < n; ++i) {
-        if (applications[i].app_name.c_str() == Get(ACTIVE_APPLICATION)) {
-            return applications[i];
+    for (std::size_t i = 0, n = this->applications.size(); i < n; ++i) {
+        if (this->applications[i].app_name == this->active_application) {
+            return this->applications[i];
         }
     }
 
-    return applications[0];  // Not found, but the list is present, so return the first item.
+    return this->applications[0];  // Not found, but the list is present, so return the first item.
 }
 
 const Application& Environment::GetApplication(std::size_t application_index) const {
-    assert(application_index < applications.size());
+    assert(application_index < this->applications.size());
 
-    return applications[application_index];
+    return this->applications[application_index];
 }
 
 Application& Environment::GetApplication(std::size_t application_index) {
-    assert(application_index < applications.size());
+    assert(application_index < this->applications.size());
 
-    return applications[application_index];
+    return this->applications[application_index];
 }
 
-bool Environment::UseOverride() const { return (override_state & OVERRIDE_FLAG_ACTIVE) != 0; }
+bool Environment::GetPerApplicationConfig() const { return this->use_per_application_configuration; }
 
-bool Environment::UseApplicationListOverrideMode() const { return (override_state & OVERRIDE_FLAG_SELECTED) != 0; }
+void Environment::SetPerApplicationConfig(bool enable) { this->use_per_application_configuration = enable; }
 
-bool Environment::UsePersistentOverrideMode() const { return (override_state & OVERRIDE_FLAG_PERSISTENT) != 0; }
-
-void Environment::SetMode(OverrideMode mode, bool enabled) {
-    switch (mode) {
-        case OVERRIDE_MODE_ACTIVE:
-            if (enabled)
-                override_state = static_cast<OverrideState>(override_state | OVERRIDE_FLAG_ACTIVE);
-            else
-                override_state = static_cast<OverrideState>(override_state & ~OVERRIDE_FLAG_ACTIVE);
-            break;
-        case OVERRIDE_MODE_LIST:
-            if (enabled)
-                override_state = static_cast<OverrideState>(override_state | OVERRIDE_FLAG_SELECTED);
-            else
-                override_state = static_cast<OverrideState>(override_state & ~OVERRIDE_FLAG_SELECTED);
-            break;
-        case OVERRIDE_MODE_PERISTENT:
-            if (enabled)
-                override_state = static_cast<OverrideState>(override_state | OVERRIDE_FLAG_PERSISTENT);
-            else
-                override_state = static_cast<OverrideState>(override_state & ~OVERRIDE_FLAG_PERSISTENT);
-            break;
-        default:
-            assert(0);
-            break;
-    }
+bool Environment::GetUseApplicationList() const {
+    return this->use_application_list && this->layers_mode != LAYERS_MODE_BY_APPLICATIONS;
 }
 
+void Environment::SetUseApplicationList(bool enable) { this->use_application_list = enable; }
+
+LayersMode Environment::GetMode() const { return this->layers_mode; }
+
+void Environment::SetMode(LayersMode mode) { this->layers_mode = mode; }
+
 void Environment::Set(LayoutState state, const QByteArray& data) {
     assert(state >= LAYOUT_FIRST && state <= LAYOUT_LAST);
-    layout_states[state] = data;
+    this->layout_states[state] = data;
 }
 
 const QByteArray& Environment::Get(LayoutState state) const {
     assert(state >= LAYOUT_FIRST && state <= LAYOUT_LAST);
-    return layout_states[state];
+    return this->layout_states[state];
 }
 
-const std::string& Environment::Get(Active active) const { return actives[active]; }
-
-void Environment::Set(Active active, const std::string& key) { actives[active] = key; }
-
 bool Environment::IsDefaultConfigurationInit(const std::string& default_configuration_filename) const {
-    for (std::size_t i = 0, n = default_configuration_filenames.size(); i < n; ++i) {
-        if (default_configuration_filenames[i] == default_configuration_filename) return true;
+    for (std::size_t i = 0, n = this->default_configuration_filenames.size(); i < n; ++i) {
+        if (this->default_configuration_filenames[i] == default_configuration_filename) {
+            return true;
+        }
     }
 
     return false;
@@ -570,7 +570,7 @@ std::string Environment::GetDefaultExecutablePath(const std::string& executable_
     static const char* DEFAULT_PATH = VKC_PLATFORM == VKC_PLATFORM_MACOS ? "/../.." : "";
 
     // Using VULKAN_SDK environement variable
-    const std::string env(paths_manager.GetPath(PATH_VULKAN_SDK));
+    const std::string env("${VULKAN_SDK}");
     if (!env.empty()) {
         static const char* TABLE[] = {
             "/Bin",  // ENVIRONMENT_WIN32
@@ -579,39 +579,41 @@ std::string Environment::GetDefaultExecutablePath(const std::string& executable_
         static_assert(countof(TABLE) == ENVIRONMENT_COUNT, "The tranlation table size doesn't match the enum number of elements");
 
         const std::string search_path = env + TABLE[VKC_ENV] + DEFAULT_PATH + executable_name.c_str();
-        const QFileInfo file_info(search_path.c_str());
+        const QFileInfo file_info(ReplaceBuiltInVariable(search_path).c_str());
         if (file_info.exists()) {
-            return file_info.absoluteFilePath().toStdString();
+            return search_path;
         }
     }
 
     // Such the default applications from package installation (Linux)
     if (VKC_PLATFORM == VKC_PLATFORM_LINUX) {
         const std::string search_path = std::string("/usr/bin") + DEFAULT_PATH + executable_name.c_str();
-        const QFileInfo file_info(search_path.c_str());
+        const QFileInfo file_info(ReplaceBuiltInVariable(search_path).c_str());
         if (file_info.exists()) {
-            return file_info.absoluteFilePath().toStdString();
+            return search_path;
         }
     } else if (VKC_PLATFORM == VKC_PLATFORM_MACOS) {
-        std::string search_path = std::string("/Applications") + executable_name.c_str();
+        const std::string search_path = std::string("/Applications") + executable_name.c_str();
+        std::string replaced_string = ReplaceBuiltInVariable(search_path);
         const QFileInfo file_info(search_path.c_str());
-        if (file_info.exists() && ExactExecutableFromAppBundle(search_path)) {
-            return search_path;
+        if (file_info.exists() && ExactExecutableFromAppBundle(replaced_string)) {
+            return replaced_string;
         }
     }
 
     // Using relative path to vkconfig in case SDK is not "installed"
     if (VKC_PLATFORM == VKC_PLATFORM_MACOS) {
-        std::string search_path = std::string("..") + DEFAULT_PATH + executable_name.c_str();
-        const QFileInfo file_info(search_path.c_str());
-        if (file_info.exists() && ExactExecutableFromAppBundle(search_path)) {
-            return search_path;
+        const std::string search_path = std::string("..") + DEFAULT_PATH + executable_name.c_str();
+        std::string replaced_string = ReplaceBuiltInVariable(search_path);
+        const QFileInfo file_info(replaced_string.c_str());
+        if (file_info.exists() && ExactExecutableFromAppBundle(replaced_string)) {
+            return replaced_string;
         }
     } else {
-        const QString search_path = QString(".") + DEFAULT_PATH + executable_name.c_str();
-        const QFileInfo file_info(search_path);
+        const std::string search_path = std::string(".") + DEFAULT_PATH + executable_name.c_str();
+        const QFileInfo file_info(ReplaceBuiltInVariable(search_path).c_str());
         if (file_info.exists()) {
-            return file_info.absoluteFilePath().toStdString();
+            return search_path;
         }
     }
 
