@@ -40,6 +40,8 @@
 #include <QDir>
 #include <QMessageBox>
 #include <QJsonArray>
+#include <QSettings>
+#include <QDateTime>
 
 #include <cassert>
 #include <string>
@@ -162,7 +164,6 @@ SettingMeta* Layer::Instantiate(SettingMetaSet& meta_set, const std::string& key
     return setting_meta;
 }
 
-/// Reports errors via a message box. This might be a bad idea?
 bool Layer::Load(const std::vector<Layer>& available_layers, const std::string& full_path_to_file, LayerType layer_type) {
     this->type = layer_type;  // Set layer type, no way to know this from the json file
 
@@ -209,6 +210,11 @@ bool Layer::Load(const std::vector<Layer>& available_layers, const std::string&
 
     const QJsonObject& json_layer_object = ReadObject(json_root_object, "layer");
 
+    std::string current_last_modified = QFileInfo(full_path_to_file.c_str()).lastModified().toString(Qt::ISODate).toStdString();
+
+    QSettings settings;
+    std::string cached_last_modified = settings.value(full_path_to_file.c_str()).toString().toStdString();
+
     this->key = ReadStringValue(json_layer_object, "name");
 
     if (this->key == "VK_LAYER_LUNARG_override") {
@@ -226,13 +232,14 @@ bool Layer::Load(const std::vector<Layer>& available_layers, const std::string&
         full_path_to_file.rfind(":/") == 0;  // Check whether the path start with ":/" for resource file paths.
 
     JsonValidator validator;
-#if defined(_DEBUG)
-    const bool should_validate = (this->api_version >= Version(1, 2, 170) && is_builtin_layer_file) || !is_builtin_layer_file;
-#else
-    const bool should_validate = !is_builtin_layer_file;
-#endif
+
+    const bool should_validate = current_last_modified != cached_last_modified;
     const bool is_valid = should_validate ? validator.Check(json_text) : true;
 
+    if (should_validate && is_valid) {
+        settings.setValue(full_path_to_file.c_str(), current_last_modified.c_str());
+    }
+
     const QJsonValue& json_library_path_value = json_layer_object.value("library_path");
     if (json_library_path_value != QJsonValue::Undefined) {
         this->binary_path = json_library_path_value.toString().toStdString();
@@ -253,6 +260,19 @@ bool Layer::Load(const std::vector<Layer>& available_layers, const std::string&
         this->url = ReadStringValue(json_layer_object, "url");
     }
 
+    if (json_layer_object.value("disable_environment") != QJsonValue::Undefined) {
+        const QJsonObject& json_env_object = json_layer_object.value("disable_environment").toObject();
+        const QStringList keys = json_env_object.keys();
+        this->disable_env = keys[0].toStdString();
+        this->disable_value = ReadStringValue(json_env_object, this->disable_env.c_str()) == "1";
+    }
+    if (json_layer_object.value("enable_environment") != QJsonValue::Undefined) {
+        const QJsonObject& json_env_object = json_layer_object.value("enable_environment").toObject();
+        const QStringList keys = json_env_object.keys();
+        this->enable_env = keys[0].toStdString();
+        this->enable_value = ReadStringValue(json_env_object, this->enable_env.c_str()) == "1";
+    }
+
     if (!is_valid && this->key != "VK_LAYER_LUNARG_override") {
         if (!is_builtin_layer_file || (is_builtin_layer_file && this->api_version >= Version(1, 2, 170))) {
             Alert::LayerInvalid(full_path_to_file.c_str(), validator.message.toStdString().c_str());
@@ -313,6 +333,170 @@ bool Layer::Load(const std::vector<Layer>& available_layers, const std::string&
     return this->IsValid();  // Not all JSON file are layer JSON valid
 }
 
+bool Layer::Load(const std::string& full_path_to_file, LayerType layer_type) {
+    this->type = layer_type;  // Set layer type, no way to know this from the json file
+
+    if (full_path_to_file.empty()) return false;
+
+    QFile file(full_path_to_file.c_str());
+    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        return false;
+    }
+
+    QString json_text = file.readAll();
+    file.close();
+
+    this->manifest_path = full_path_to_file;
+
+    // Convert the text to a JSON document & validate it.
+    // It does need to be a valid json formatted file.
+    QJsonParseError json_parse_error;
+    const QJsonDocument& json_document = QJsonDocument::fromJson(json_text.toUtf8(), &json_parse_error);
+    if (json_parse_error.error != QJsonParseError::NoError) {
+        return false;
+    }
+
+    // Make sure it's not empty
+    if (json_document.isNull() || json_document.isEmpty()) {
+        return false;
+    }
+
+    // First check it's a layer manifest, ignore otherwise.
+    const QJsonObject& json_root_object = json_document.object();
+    if (json_root_object.value("file_format_version") == QJsonValue::Undefined) {
+        return false;  // Not a layer JSON file
+    }
+    if (json_root_object.value("layer") == QJsonValue::Undefined) {
+        return false;  // Not a layer JSON file
+    }
+
+    this->file_format_version = ReadVersionValue(json_root_object, "file_format_version");
+    if (this->file_format_version.GetMajor() > 1) {
+        Alert::LayerInvalid(full_path_to_file.c_str(),
+                            format("Unsupported layer file format: %s", this->file_format_version.str().c_str()).c_str());
+        return false;
+    }
+
+    const QJsonObject& json_layer_object = ReadObject(json_root_object, "layer");
+
+    std::string current_last_modified = QFileInfo(full_path_to_file.c_str()).lastModified().toString(Qt::ISODate).toStdString();
+
+    QSettings settings;
+    std::string cached_last_modified = settings.value(full_path_to_file.c_str()).toString().toStdString();
+
+    this->key = ReadStringValue(json_layer_object, "name");
+
+    if (this->key == "VK_LAYER_LUNARG_override") {
+        return false;
+    }
+
+    this->api_version = ReadVersionValue(json_layer_object, "api_version");
+
+    const bool is_builtin_layer_file =
+        full_path_to_file.rfind(":/") == 0;  // Check whether the path start with ":/" for resource file paths.
+
+    JsonValidator validator;
+
+    const bool should_validate = current_last_modified != cached_last_modified;
+    const bool is_valid = should_validate ? validator.Check(json_text) : true;
+
+    if (should_validate && is_valid) {
+        settings.setValue(full_path_to_file.c_str(), current_last_modified.c_str());
+    }
+
+    const QJsonValue& json_library_path_value = json_layer_object.value("library_path");
+    if (json_library_path_value != QJsonValue::Undefined) {
+        this->binary_path = json_library_path_value.toString().toStdString();
+    }
+
+    this->implementation_version = ReadStringValue(json_layer_object, "implementation_version");
+    if (json_layer_object.value("status") != QJsonValue::Undefined) {
+        this->status = GetStatusType(ReadStringValue(json_layer_object, "status").c_str());
+    }
+    if (json_layer_object.value("platforms") != QJsonValue::Undefined) {
+        this->platforms = GetPlatformFlags(ReadStringArray(json_layer_object, "platforms"));
+    }
+    this->description = ReadStringValue(json_layer_object, "description");
+    if (json_layer_object.value("introduction") != QJsonValue::Undefined) {
+        this->introduction = ReadStringValue(json_layer_object, "introduction");
+    }
+    if (json_layer_object.value("url") != QJsonValue::Undefined) {
+        this->url = ReadStringValue(json_layer_object, "url");
+    }
+
+    if (json_layer_object.value("disable_environment") != QJsonValue::Undefined) {
+        const QJsonObject& json_env_object = json_layer_object.value("disable_environment").toObject();
+        const QStringList keys = json_env_object.keys();
+        this->disable_env = keys[0].toStdString();
+        this->disable_value = ReadStringValue(json_env_object, this->disable_env.c_str()) == "1";
+    }
+    if (json_layer_object.value("enable_environment") != QJsonValue::Undefined) {
+        const QJsonObject& json_env_object = json_layer_object.value("enable_environment").toObject();
+        const QStringList keys = json_env_object.keys();
+        this->enable_env = keys[0].toStdString();
+        this->enable_value = ReadStringValue(json_env_object, this->enable_env.c_str()) == "1";
+    }
+
+    if (!is_valid && this->key != "VK_LAYER_LUNARG_override") {
+        if (!is_builtin_layer_file || (is_builtin_layer_file && this->api_version >= Version(1, 2, 170))) {
+            Alert::LayerInvalid(full_path_to_file.c_str(), validator.message.toStdString().c_str());
+            return false;
+        }
+    }
+
+    const QJsonValue& json_features_value = json_layer_object.value("features");
+    if (json_features_value != QJsonValue::Undefined) {
+        const QJsonObject& json_features_object = json_features_value.toObject();
+
+        // Load layer settings
+        const QJsonValue& json_settings_value = json_features_object.value("settings");
+        if (json_settings_value != QJsonValue::Undefined) {
+            AddSettingsSet(this->settings, nullptr, json_settings_value);
+        }
+
+        // Load layer presets
+        const QJsonValue& json_presets_value = json_features_object.value("presets");
+        if (json_presets_value != QJsonValue::Undefined) {
+            assert(json_presets_value.isArray());
+            const QJsonArray& json_preset_array = json_presets_value.toArray();
+            for (int preset_index = 0, preset_count = json_preset_array.size(); preset_index < preset_count; ++preset_index) {
+                const QJsonObject& json_preset_object = json_preset_array[preset_index].toObject();
+
+                LayerPreset preset;
+                preset.platform_flags = this->platforms;
+                preset.status = this->status;
+                LoadMetaHeader(preset, json_preset_object);
+
+                const QJsonArray& json_setting_array = ReadArray(json_preset_object, "settings");
+                for (int setting_index = 0, setting_count = json_setting_array.size(); setting_index < setting_count;
+                     ++setting_index) {
+                    AddSettingData((SettingDataSet&)preset.settings, json_setting_array[setting_index]);
+                }
+
+                this->presets.push_back(preset);
+            }
+        }
+    }
+
+    // Override old built-in layer settings
+    if (!is_builtin_layer_file && this->api_version <= Version(1, 2, 176)) {
+        const std::string path = GetBuiltinFolder(this->api_version) + "/" + this->key + ".json";
+
+        Layer default_layer;
+        if (default_layer.Load(path, this->type)) {
+            this->introduction = default_layer.introduction;
+            this->url = default_layer.url;
+            this->platforms = default_layer.platforms;
+            this->status = default_layer.status;
+            std::swap(this->settings, default_layer.settings);
+            std::swap(this->presets, default_layer.presets);
+            this->memory = default_layer.memory;
+        }
+    }
+
+    return this->IsValid();  // Not all JSON file are layer JSON valid
+}
+
 void CollectDefaultSettingData(const SettingMetaSet& meta_set, SettingDataSet& data_set) {
     for (std::size_t i = 0, n = meta_set.size(); i < n; ++i) {
         SettingMeta* setting_meta = meta_set[i];
@@ -362,7 +546,6 @@ void Layer::AddSettingsSet(SettingMetaSet& settings, const SettingMeta* parent,
                     const QJsonObject& json_root_object = doc.object();
                     const std::string schema = json_root_object.value("$schema").toString().toStdString();
                     if (schema.find("https://schema.khronos.org/vulkan/profiles") == std::string::npos) {
-                        Alert::FileNotProfile(value.c_str());
                         return;
                     }
                 }
