@@ -22,15 +22,17 @@
 
 #include <gtest/gtest.h>
 
+static const std::vector<std::string> SUPPORTED_CONFIG_FILES = {"_1_0_0"};
+
 TEST(test_layer_manager, load_only_layer_json) {
-    PathManager paths("");
+    PathManager paths("", SUPPORTED_CONFIG_FILES);
     Environment environment(paths);
     environment.Reset(Environment::DEFAULT);
 
     LayerManager layer_manager(environment);
     layer_manager.LoadLayersFromPath(":/");
 
-    EXPECT_EQ(10, layer_manager.available_layers.size());
+    EXPECT_EQ(10, layer_manager.selected_layers.size());
 
     environment.Reset(Environment::SYSTEM);  // Don't change the system settings on exit
 }
